"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceCollection = void 0;
var Errors_1 = require("../Errors");
var Lifetime_1 = require("../Lifetime");
var ServiceProvider_1 = require("../ServiceProvider");
var utils_1 = require("../utils");
var mockUtils_1 = require("./mockUtils");
var ServiceCollection = /** @class */ (function () {
    function ServiceCollection(other, lifetime) {
        var _a;
        Object.defineProperty(this, "self", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.self = this;
        this.lifetimes = (_a = other === null || other === void 0 ? void 0 : other.lifetimes) !== null && _a !== void 0 ? _a : {};
        if (lifetime)
            this.lifetimes[lifetime.name] = lifetime;
    }
    Object.defineProperty(ServiceCollection.prototype, "addStateful", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (name, Dependency) {
            var last = Lifetime_1.Lifetime.dummy("".concat(name, "@constructor"));
            var factory = this.extractStatefulFactory(Dependency);
            function next(scope) {
                scope[name] = scope[name] || 1;
                return scope[name]++;
            }
            return this.add(Lifetime_1.Transient, name, {
                factory: function (_, context) {
                    var _a;
                    var _b = (_a = context.lastSingleton) !== null && _a !== void 0 ? _a : {}, isSingleton = _b.isSingleton, lastName = _b.name;
                    var singleton = lastName ? "".concat(String(lastName), "@") : '';
                    var track = Lifetime_1.Lifetime.dummy("".concat(singleton).concat(name, "@creator#").concat(next(context.scope)), isSingleton);
                    return {
                        create: function (props) {
                            var lifetimes = [context.depth && last, track].filter(function (e) { return e; });
                            return context.enter(lifetimes, function () { return factory(context.proxy, props, context); });
                        },
                    };
                },
            });
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (name, Dependency) {
            return this.add(Lifetime_1.Singleton, name, Dependency);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (name, Dependency) {
            return this.add(Lifetime_1.Scoped, name, Dependency);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (name, Dependency) {
            return this.add(Lifetime_1.Transient, name, Dependency);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "add", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, name, Dependency) {
            if (name in this.lifetimes)
                throw new Errors_1.DuplicateDependencyError(name);
            return new ServiceCollection(this.self, new Lifetime(name, this.extractFactory(Dependency)));
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "get", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            return this.lifetimes[(0, utils_1.extractSelector)(selector)];
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "remove", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            delete this.lifetimes[(0, utils_1.extractSelector)(selector)];
            return new ServiceCollection(this.self);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "build", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return new ServiceProvider_1.ServiceProvider(this.cloneLifetimes());
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "buildMock", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (mock) {
            if (mock === void 0) { mock = {}; }
            return (0, mockUtils_1.proxyLifetimes)(this, mock);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "extractStatefulFactory", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Option) {
            return typeof Option === 'function'
                ? function (provider, props) { return new Option(provider, props); }
                : Option.factory;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "extractFactory", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Option) {
            return typeof Option === 'function'
                ? function (provider) { return new Option(provider); }
                : Option.factory;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "cloneLifetimes", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return Object.entries(this.lifetimes)
                .map(function (_a) {
                var key = _a[0], value = _a[1];
                return [key, value.clone()];
            })
                .reduce(function (acc, _a) {
                var key = _a[0], value = _a[1];
                acc[key] = value;
                return acc;
            }, {});
        }
    });
    return ServiceCollection;
}());
exports.ServiceCollection = ServiceCollection;
